/*
Copyright © 2020 NAME HERE <EMAIL ADDRESS>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package cmd

import (
	"fmt"

	"github.com/jmoiron/sqlx"
	"github.com/spf13/cobra"
	"gitlab-ce.k8s.tools.vchangyi.com/cdp/cy-sdk-go/exception"
	"gitlab-ce.k8s.tools.vchangyi.com/cdp/cy-sdk-go/wechat"

	// mysql
	_ "github.com/go-sql-driver/mysql"
)

var partnerStockID string
var partnerMchID string
var sourceMchID string
var sourceDsn string

// partnerCmd represents the partner command
var partnerCmd = &cobra.Command{
	Use:   "partner",
	Short: "建立委托营销关系",
	Long:  `建立委托营销关系：参数商户号和批次号必须指定`,
	Run: func(cmd *cobra.Command, args []string) {
		db := sqlx.MustConnect("mysql", sourceDsn)
		cfg := &wechat.ClientV3Config{}
		exception.Assert(db.Get(cfg, "select mch_id, serial_no, pemkey, pemcert, appid, api_secret, service_mch_id from hwj_mch_conf where mch_id=?", sourceMchID))
		wc, err := wechat.NewWechatClientV3WithPem(cfg, nil)
		exception.Assert(err)
		if partnerStockID != "" && partnerMchID != "" {
			result, err := wc.PartnerShip(wechat.StockTypeBusiFavor, partnerMchID, partnerStockID)
			if err != nil {
				fmt.Println("建立失败", err)
			} else {
				fmt.Println("建立成功", result)
			}
		}
	},
}

func init() {
	rootCmd.AddCommand(partnerCmd)

	// Here you will define your flags and configuration settings.
	partnerCmd.Flags().StringVar(&sourceDsn, "dsn", "", "批次号配置数据库")
	partnerCmd.Flags().StringVar(&sourceMchID, "source-mchid", "1561018721", "批次号所属商户")
	partnerCmd.Flags().StringVar(&partnerStockID, "partner-stock", "", "批次号")
	partnerCmd.Flags().StringVar(&partnerMchID, "partner-mchid", "", "授权商户号")
}
