package tkserver

import (
	context "context"
	"errors"

	"github.com/douyu/jupiter"
	"github.com/douyu/jupiter/pkg/server/xgrpc"
	"github.com/getsentry/sentry-go"
	"github.com/spf13/viper"
	"gitlab-ce.k8s.tools.vchangyi.com/cdp/cy-sdk-go/exception"
	"gitlab-ce.k8s.tools.vchangyi.com/cdp/cy-sdk-go/wechat/cache"
	"gitlab-ce.k8s.tools.vchangyi.com/cdp/cy-sdk-go/wechat/client"
	"gitlab-ce.k8s.tools.vchangyi.com/cdp/cy-sdk-go/wechat/crypt"
)

type tkServerEngine struct {
	jupiter.Application
	apps   map[string]*client.Client
	apikey string
}

func (engine *tkServerEngine) serverRPC() error {
	cfg := xgrpc.DefaultConfig()
	cfg.Host = viper.GetString("tkserver.addr")
	cfg.Port = viper.GetInt("tkserver.port")
	server := cfg.Build()
	RegisterTKServerServer(server.Server, &impTKServer{engine: engine})
	return engine.Serve(server)
}

// NewEngine 创建服务
func NewEngine() *jupiter.Application {
	engine := &tkServerEngine{
		apps:   make(map[string]*client.Client),
		apikey: viper.GetString("apikey"),
	}
	apps := viper.GetStringMapString("apps")
	for appid, appsecret := range apps {
		engine.apps[appid] = client.NewClient(&crypt.WechatConfig{
			AppID:     appid,
			AppSecret: appsecret,
		}, cache.NewJsonFileCache(appid+".json"))
	}
	exception.Assert(engine.Startup(engine.serverRPC))
	return &engine.Application
}

type impTKServer struct {
	engine *tkServerEngine
}

// ErrAuth 未授权
var ErrAuth = errors.New("noauth")

// ErrAppID 应用不支持
var ErrAppID = errors.New("appid")

func (g impTKServer) AccessToken(context context.Context, request *AccessTokenRequest) (*AccessTokenReply, error) {
	if request.Apikey != g.engine.apikey {
		return nil, ErrAuth
	}

	c, ok := g.engine.apps[request.Appid]
	if !ok {
		return nil, ErrAppID
	}
	reply := &AccessTokenReply{}
	var err error
	if reply.Token, err = c.GetAccessToken(); err != nil {
		sentry.CaptureException(err)
		return nil, err
	}

	if reply.Token == request.Expired {
		c.ExpireToken()
		if reply.Token, err = c.GetAccessToken(); err != nil {
			sentry.CaptureException(err)
			return nil, err
		}
	}
	return reply, nil
}
