package wechat

import (
	"bytes"
	"fmt"
	"io/ioutil"
	"log"
	"os"
	"path/filepath"
	"strings"
	"testing"

	"golang.org/x/text/encoding/simplifiedchinese"
	"golang.org/x/text/transform"
)

/*
func TestGetCopuonByUserWithAvailableMch(t *testing.T) {
	//c, err := NewWechatClientV3("1530390001", "wx80436b78f0797bda", "cert/13_WECHAT_PAY_private_key_20190412111528.pem", "5D172589C34A29FB09FBA10CFCC8A7ABC1C386C2")
	c, err := NewWechatClientV3("1532273371", "wxe11d047c889f9c83", "test/2_WECHAT_PAY_private_key_20190801122648.pem", "73B7538A6C9DC60040B75FDD084F84D7E8655555", 0)

	if err != nil {
		t.Fatal(err)
	}
	resp, err := c.GetCopuonByUserWithAvlMch("o_aHr4gpiGuuykqR0BSMb761whhY")
	b, _ := json.Marshal(resp)
	log.Println(string(b))
}

*/

/*
func TestSendCoupon(t *testing.T) {
	//c, err := NewWechatClientV3("1530390001", "wx80436b78f0797bda", "cert/13_WECHAT_PAY_private_key_20190412111528.pem", "5D172589C34A29FB09FBA10CFCC8A7ABC1C386C2")
	c, err := NewWechatClientV3("1532273371", "wxe11d047c889f9c83", "test/2_WECHAT_PAY_private_key_20190801122648.pem", "73B7538A6C9DC60040B75FDD084F84D7E8655555", 0)

	//c, err := NewWechatClientV3("1530183331", "", "cert/apiclient_key.pem", "5CFB0A801629141DD81DD8BA2A97E3EF2AD23C03")
	if err != nil {
		t.Fatal(err)
	}
	rno, coupon, err := c.SendCoupon("o_aHr4gpiGuuykqR0BSMb761whhY", "9581999", "1530183331", c.appid)
	t.Logf("%s %s %v", rno, coupon, err)
	//rno, coupon, err = c.SendCoupon("o_aHr4gpiGuuykqR0BSMb761whhY", "9623055", "1530183331", c.appid)
	t.Logf("%s %s %v", rno, coupon, err)
}
*/

/*
func TestQueryCoupon(t *testing.T) {
	c, _ := NewWechatClientV3("1532229411", "wx5359f11158e1ca89", "cert/16_WECHAT_PAY_private_key_20190530102451.pem", "1286B0853EC97355ABE11D3C1D6EEC5EB0FD97A4", 0)
	stock, err := c.GetStockByID("9579481", "1530183331")
	if err != nil {
		t.Fatal(err)
	}
	b, _ := json.Marshal(stock)
	log.Println(string(b))
}
*/

func Utf8ToGbk(s []byte) ([]byte, error) {
	reader := transform.NewReader(bytes.NewReader(s), simplifiedchinese.GBK.NewEncoder())
	d, e := ioutil.ReadAll(reader)
	if e != nil {
		return nil, e
	}
	return d, nil
}

func readfile(fpath string) ([]byte, error) {
	f, err := os.Open(fpath)
	if err != nil {
		return nil, err
	}
	data, err := ioutil.ReadAll(f)
	if err != nil {
		return nil, err
	}
	return data, err
}

func TestLoadStockUserFlow(t *testing.T) {
	path := `c:\Users\edz\Documents\src\cdp-coupon-service\cert\apiclient_key.pem`
	stockbath := `c:\Users\edz\Documents\stock_userflow\`
	stockpath := `c:\Users\edz\Documents\stock_userflow\qiaqiastocks_t.csv`
	filterpath := `c:\Users\edz\Documents\stock_userflow\qiaqiastocks_filter`
	laterpath := `c:\Users\edz\Documents\stock_userflow\qiaqiastocks_laster`
	c, err := NewWechatClientV3("1530183331", "", "wx80436b78f0797bda", path, "5CFB0A801629141DD81DD8BA2A97E3EF2AD23C03", nil)
	if err != nil {
		log.Fatalf("create wechat client err %s", err)
		return
	}
	data, err := readfile(stockpath)
	if err != nil {
		log.Fatalf("read stock path err %s", err)
		return
	}
	stockidstr := string(data)
	stockidstr = strings.Replace(stockidstr, "\r\n", "", -1)
	stockids := strings.Split(stockidstr, ",")
	filter := make([]string, 0, 0)
	filterStr := ""
	data, err = readfile(filterpath)
	if err == nil {
		filterStr = string(data)
		filter = strings.Split(string(data), ",")
	}
	later := make([]string, 0, 0)
	lasterStr := ""
	data, err = readfile(laterpath)
	if err == nil {
		lasterStr = string(data)
		later = strings.Split(string(data), ",")
	}
	stock_report := "批次id,发放数,面额,核销,"
	for _, stockid := range stockids {
		if len(stockid) == 0 {
			continue
		}
		stock, err := c.GetStockByID(stockid, "1530183331")
		if err != nil {
			log.Println(fmt.Sprintf("%s GetStockByID err:%s", stockid, err))
		} else {
			maxCn := stock.StockUseRule.MaxCoupons
			cnAmt := stock.StockUseRule.FixedNormalCoupon.CouponAmount
			use_flow_cnt := "账单文件不存在"
			if !strings.Contains(filterStr, stockid) && !strings.Contains(lasterStr, stockid) {
				filename := filepath.Join(stockbath, stockid)
				if err = c.DownloadStockUse(stockid, filename, filename); err != nil {
					if strings.Contains(err.Error(), use_flow_cnt) {
						filter = append(filter, stockid)
					} else if strings.Contains(err.Error(), "请在批次过期后第二天10点后下载") {
						later = append(later, stockid)
						use_flow_cnt = "请在批次过期后第二天10点后下载"
					} else {
						continue
					}
				} else if use_flow_content, err := readfile(filename); err == nil {
					idx := strings.Index(string(use_flow_content), "总条数")
					use_flow_cnt = string(use_flow_content)[idx+12:]
					use_flow_cnt = strings.Replace(use_flow_cnt, "\r\n", "", -1)
				} else {
					continue
				}
			}
			stock_report = fmt.Sprintf("%s\r\n %s,%d,%d,%s", stock_report, stockid, maxCn, cnAmt, use_flow_cnt)
			log.Printf("%s,%d,%d,%s\n", stockid, maxCn, cnAmt, use_flow_cnt)
		}
	}
	data, _ = Utf8ToGbk([]byte(stock_report))
	if err = ioutil.WriteFile(stockbath+"report.csv", data, 0644); err != nil {
		log.Printf("创建report失败:%s\n", err)
	}
	if err = ioutil.WriteFile(filterpath, []byte(strings.Join(filter, ",")), 0644); err != nil {
		log.Printf("写 filter 失败%s", err)
	}
	if err = ioutil.WriteFile(laterpath, []byte(strings.Join(later, ",")), 0644); err != nil {
		log.Printf("写 later 失败%s", err)
	}
}

type ConfigNotifyUrlReq struct {
	Mchid     string `json:"mchid"`
	NotifyUrl string `json:"notify_url"`
	Switch    bool   `json:"switch"`
}

func TestConfigNotifyUrl(t *testing.T) {
	url := "https://api.mch.weixin.qq.com/v3/marketing/favor/callbacks"
	path := `/Users/invictus/Downloads/cert/apiclient_key.pem`
	c, err := NewWechatClientV3("1530183331", "", "", path, "5CFB0A801629141DD81DD8BA2A97E3EF2AD23C03", nil)
	req := ConfigNotifyUrlReq{
		Mchid:     "1530183331",
		NotifyUrl: "https://wechat-notify.vchangyi.com/coupon/onuse",
		Switch:    true,
	}
	rdata, err := c.DoPost(url, req)
	if err != nil {
		log.Println(err)
	} else {
		log.Println(string(rdata))
	}
}
