package wechat

import (
	"bytes"
	"encoding/json"
	"fmt"
	"io/ioutil"
	"net/http"
)

// SendQYWechatRobotMarkDown 发送企业微信机器人消息
func SendQYWechatRobotMarkDown(link string, content string) error {
	msg := map[string]interface{}{
		"markdown": map[string]string{
			"content": content,
		},
		"msgtype": "markdown",
	}
	b, err := json.Marshal(msg)
	if err != nil {
		return err
	}

	resp, err := http.Post(link, "application/json", bytes.NewBuffer(b))
	if err != nil {
		return err
	}

	if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("%d: %s", resp.StatusCode, resp.Status)
	}
	qwResp := struct {
		ErrCode int    `json:"errcode"`
		ErrMsg  string `json:"errmsg"`
	}{}
	rb, err := ioutil.ReadAll(resp.Body)
	defer resp.Body.Close()
	if err != nil {
		return err
	}
	if err = json.Unmarshal(rb, &qwResp); err != nil {
		return err
	}
	if qwResp.ErrCode != 0 {
		err = fmt.Errorf("qw response:%s", string(rb))
	}
	return err
}
